# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-TODAY Pexego (<www.pexego.es>). All Rights Reserved
#    $Omar Castiñeira Saavedra$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

"""creates a delivery line in pirchase order line"""

import time
from datetime import date,timedelta
import wizard
import pooler
from tools.translate import _

delivery_form = """<?xml version="1.0"?>
<form string="Create deliveries">
    <separator colspan="4" string="Delivery Method" />
    <field name="carrier_id" />
</form>
"""

delivery_fields = {
    'carrier_id' : {'string':'Delivery Method', 'type':'many2one', 'relation': 'delivery.carrier','required':True}
}

def _delivery_default(self, cr, uid, data, context):
    """sets default values"""
    order_obj = pooler.get_pool(cr.dbname).get('purchase.order')
    order = order_obj.browse(cr,uid,data['ids'],)[0]
    if not order.state in ('draft'):
        raise wizard.except_wizard(_('Order not in draft state !'), _('The order state have to be draft to add delivery lines.'))
    carrier_id = order.partner_id.property_delivery_carrier.id or (order.carrier_id and order.carrier_id.id or False)
    return {'carrier_id': carrier_id}

def _delivery_set(self, cr, uid, data, context):
    """creates the purchase line"""
    order_obj = pooler.get_pool(cr.dbname).get('purchase.order')
    line_obj = pooler.get_pool(cr.dbname).get('purchase.order.line')
    order_objs = order_obj.browse(cr, uid, data['ids'], context)

    for order in order_objs:
        grid_id = pooler.get_pool(cr.dbname).get('delivery.carrier').grid_get(cr, uid, [data['form']['carrier_id']],order.partner_address_id.id)
        if not grid_id:
            raise wizard.except_wizard(_('No grid avaible !'), _('No grid matching for this carrier !'))
        grid_obj = pooler.get_pool(cr.dbname).get('delivery.grid')
        grid = grid_obj.browse(cr, uid, [grid_id])[0]

        line_obj.create(cr, uid, {
            'order_id': order.id,
            'name': grid.carrier_id.name,
            'product_qty': 1,
            'product_uom': grid.carrier_id.product_id.uom_id.id,
            'product_id': grid.carrier_id.product_id.id,
            'price_base': grid_obj.get_price(cr, uid, grid.id, order, time.strftime('%Y-%m-%d'), context={'object':'purchase'}),
            'price_unit': grid_obj.get_price(cr, uid, grid.id, order, time.strftime('%Y-%m-%d'), context={'object':'purchase'}),
            'taxes_id': [(6,0,[ x.id for x in grid.carrier_id.product_id.taxes_id])],
            'date_planned':(date.today() + timedelta(days=1)).strftime('%Y-%m-%d'),
            'shipment_line': True
            })

        order_obj.write(cr, uid, [order.id], {'carrier_id': data['form']['carrier_id']})

    return {}

class make_delivery(wizard.interface):
    """creates a delivery line in purchase order line"""
    states = {
        'init' : {
            'actions' : [_delivery_default],
            'result' : {'type' : 'form', 'arch' : delivery_form, 'fields' : delivery_fields, 'state' : [('end', 'Cancel'),('delivery', 'Create delivery line') ]}
        },
        'delivery' : {
            'actions' : [_delivery_set],
            'result' : {'type' : 'state', 'state' : 'end'}
        },
    }

make_delivery("cc_purchase_sale_delivery.delivery.purchase.order")

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

